@@ -10,10 +10,12 @@
 """
 This allows to manage showfunctionality to all data types.
 """
+import pathlib
+
 import click
 
-from aiida.cmdline.params.options.multivalue import MultipleValueOption
 from aiida.cmdline.params import options
+from aiida.cmdline.params.options.multivalue import MultipleValueOption
 from aiida.cmdline.utils import echo
 from aiida.common.exceptions import MultipleObjectsError
 
@@ -53,8 +55,8 @@ def _show_jmol(exec_name, trajectory_list, **kwargs):
     """
     Plugin for jmol
     """
-    import tempfile
     import subprocess
+    import tempfile
 
     # pylint: disable=protected-access
     with tempfile.NamedTemporaryFile(mode='w+b') as handle:
@@ -66,13 +68,10 @@ def _show_jmol(exec_name, trajectory_list, **kwargs):
             subprocess.check_output([exec_name, handle.name])
         except subprocess.CalledProcessError:
             # The program died: just print a message
-            echo.echo_info(f'the call to {exec_name} ended with an error.')
+            echo.echo_error(f'the call to {exec_name} ended with an error.')
         except OSError as err:
             if err.errno == 2:
-                echo.echo_critical(
-                    "No executable '{}' found. Add to the path, "
-                    'or try with an absolute path.'.format(exec_name)
-                )
+                echo.echo_critical(f"No executable '{exec_name}' found. Add to the path, or try with an absolute path.")
             else:
                 raise
 
@@ -81,8 +80,8 @@ def _show_xcrysden(exec_name, object_list, **kwargs):
     """
     Plugin for xcrysden
     """
-    import tempfile
     import subprocess
+    import tempfile
 
     if len(object_list) > 1:
         raise MultipleObjectsError('Visualization of multiple trajectories is not implemented')
@@ -97,13 +96,10 @@ def _show_xcrysden(exec_name, object_list, **kwargs):
             subprocess.check_output([exec_name, '--xsf', tmpf.name])
         except subprocess.CalledProcessError:
             # The program died: just print a message
-            echo.echo_info(f'the call to {exec_name} ended with an error.')
+            echo.echo_error(f'the call to {exec_name} ended with an error.')
         except OSError as err:
             if err.errno == 2:
-                echo.echo_critical(
-                    "No executable '{}' found. Add to the path, "
-                    'or try with an absolute path.'.format(exec_name)
-                )
+                echo.echo_critical(f"No executable '{exec_name}' found. Add to the path, or try with an absolute path.")
             else:
                 raise
 
@@ -146,8 +142,8 @@ def _show_vesta(exec_name, structure_list):
     at Kyoto University in the group of Prof. Isao Tanaka's lab
 
     """
-    import tempfile
     import subprocess
+    import tempfile
 
     # pylint: disable=protected-access
     with tempfile.NamedTemporaryFile(mode='w+b', suffix='.cif') as tmpf:
@@ -159,13 +155,10 @@ def _show_vesta(exec_name, structure_list):
             subprocess.check_output([exec_name, tmpf.name])
         except subprocess.CalledProcessError:
             # The program died: just print a message
-            echo.echo_info(f'the call to {exec_name} ended with an error.')
+            echo.echo_error(f'the call to {exec_name} ended with an error.')
         except OSError as err:
             if err.errno == 2:
-                echo.echo_critical(
-                    "No executable '{}' found. Add to the path, "
-                    'or try with an absolute path.'.format(exec_name)
-                )
+                echo.echo_critical(f"No executable '{exec_name}' found. Add to the path, or try with an absolute path.")
             else:
                 raise
 
@@ -174,8 +167,8 @@ def _show_vmd(exec_name, structure_list):
     """
     Plugin for vmd
     """
-    import tempfile
     import subprocess
+    import tempfile
 
     if len(structure_list) > 1:
         raise MultipleObjectsError('Visualization of multiple objects is not implemented')
@@ -190,13 +183,10 @@ def _show_vmd(exec_name, structure_list):
             subprocess.check_output([exec_name, tmpf.name])
         except subprocess.CalledProcessError:
             # The program died: just print a message
-            echo.echo_info(f'the call to {exec_name} ended with an error.')
+            echo.echo_error(f'the call to {exec_name} ended with an error.')
         except OSError as err:
             if err.errno == 2:
-                echo.echo_critical(
-                    "No executable '{}' found. Add to the path, "
-                    'or try with an absolute path.'.format(exec_name)
-                )
+                echo.echo_critical(f"No executable '{exec_name}' found. Add to the path, or try with an absolute path.")
             else:
                 raise
 
@@ -205,38 +195,39 @@ def _show_xmgrace(exec_name, list_bands):
     """
     Plugin for showing the bands with the XMGrace plotting software.
     """
-    import sys
     import subprocess
+    import sys
     import tempfile
+
     from aiida.orm.nodes.data.array.bands import MAX_NUM_AGR_COLORS
 
     list_files = []
     current_band_number = 0
-    for iband, bnds in enumerate(list_bands):
-        # extract number of bands
-        nbnds = bnds.get_bands().shape[1]
-        # pylint: disable=protected-access
-        text, _ = bnds._exportcontent(
-            'agr', setnumber_offset=current_band_number, color_number=(iband + 1 % MAX_NUM_AGR_COLORS)
-        )
-        # write a tempfile
-        tempf = tempfile.NamedTemporaryFile('w+b', suffix='.agr')
-        tempf.write(text)
-        tempf.flush()
-        list_files.append(tempf)
-        # update the number of bands already plotted
-        current_band_number += nbnds
 
-    try:
-        subprocess.check_output([exec_name] + [f.name for f in list_files])
-    except subprocess.CalledProcessError:
-        print(f'Note: the call to {exec_name} ended with an error.')
-    except OSError as err:
-        if err.errno == 2:
-            print(f"No executable '{exec_name}' found. Add to the path, or try with an absolute path.")
-            sys.exit(1)
-        else:
-            raise
-    finally:
-        for fhandle in list_files:
-            fhandle.close()
+    with tempfile.TemporaryDirectory() as tmpdir:
+
+        dirpath = pathlib.Path(tmpdir)
+
+        for iband, bnds in enumerate(list_bands):
+            # extract number of bands
+            nbnds = bnds.get_bands().shape[1]
+            text, _ = bnds._exportcontent(  # pylint: disable=protected-access
+                'agr', setnumber_offset=current_band_number, color_number=(iband + 1 % MAX_NUM_AGR_COLORS)
+            )
+            # write a tempfile
+            filepath = dirpath / f'{iband}.agr'
+            filepath.write_bytes(text)
+            list_files.append(str(filepath))
+            # update the number of bands already plotted
+            current_band_number += nbnds
+
+        try:
+            subprocess.check_output([exec_name] + [str(filepath) for filepath in list_files])
+        except subprocess.CalledProcessError:
+            print(f'Note: the call to {exec_name} ended with an error.')
+        except OSError as err:
+            if err.errno == 2:
+                print(f"No executable '{exec_name}' found. Add to the path, or try with an absolute path.")
+                sys.exit(1)
+            else:
+                raise
