@@ -9,17 +9,14 @@
 ###########################################################################
 """`verdi run` command."""
 import contextlib
-import os
-import functools
+import pathlib
 import sys
-import warnings
 
 import click
 
 from aiida.cmdline.commands.cmd_verdi import verdi
 from aiida.cmdline.params.options.multivalue import MultipleValueOption
-from aiida.cmdline.utils import decorators, echo
-from aiida.common.warnings import AiidaDeprecationWarning
+from aiida.cmdline.utils import decorators
 
 
 @contextlib.contextmanager
@@ -31,7 +28,7 @@ def update_environment(argv):
         _argv = sys.argv[:]
 
         # Add the current working directory to the path, such that local modules can be imported
-        sys.path.append(os.getcwd())
+        sys.path.append(pathlib.Path.cwd().resolve())
         sys.argv = argv[:]
         yield
     finally:
@@ -40,20 +37,20 @@ def update_environment(argv):
         sys.path = _path
 
 
-def validate_entrypoint_string(ctx, param, value):  # pylint: disable=unused-argument,invalid-name
+def validate_entry_point_strings(_, __, value):
     """Validate that `value` is a valid entrypoint string."""
     from aiida.orm import autogroup
 
     try:
-        autogroup.Autogroup.validate(value)
-    except Exception as exc:
-        raise click.BadParameter(f'{str(exc)} ({value})')
+        autogroup.AutogroupManager.validate(value)
+    except (TypeError, ValueError) as exc:
+        raise click.BadParameter(f'{str(exc)}: `{value}`')
 
     return value
 
 
 @verdi.command('run', context_settings=dict(ignore_unknown_options=True,))
-@click.argument('scriptname', type=click.STRING)
+@click.argument('filepath', type=click.Path(exists=True, readable=True, dir_okay=False, path_type=pathlib.Path))
 @click.argument('varargs', nargs=-1, type=click.UNPROCESSED)
 @click.option('--auto-group', is_flag=True, help='Enables the autogrouping')
 @click.option(
@@ -64,42 +61,37 @@ def validate_entrypoint_string(ctx, param, value):  # pylint: disable=unused-arg
     help='Specify the prefix of the label of the auto group (numbers might be automatically '
     'appended to generate unique names per run).'
 )
-@click.option(
-    '-n',
-    '--group-name',
-    type=click.STRING,
-    required=False,
-    help='Specify the name of the auto group [DEPRECATED, USE --auto-group-label-prefix instead]. '
-    'This also enables auto-grouping.'
-)
 @click.option(
     '-e',
     '--exclude',
+    type=str,
     cls=MultipleValueOption,
     default=None,
     help='Exclude these classes from auto grouping (use full entrypoint strings).',
-    callback=functools.partial(validate_entrypoint_string)
+    callback=validate_entry_point_strings
 )
 @click.option(
     '-i',
     '--include',
+    type=str,
     cls=MultipleValueOption,
     default=None,
-    help='Include these classes from auto grouping  (use full entrypoint strings or "all").',
-    callback=validate_entrypoint_string
+    help='Include these classes from auto grouping (use full entrypoint strings or "all").',
+    callback=validate_entry_point_strings
 )
 @decorators.with_dbenv()
-def run(scriptname, varargs, auto_group, auto_group_label_prefix, group_name, exclude, include):
-    # pylint: disable=too-many-arguments,exec-used
+def run(filepath, varargs, auto_group, auto_group_label_prefix, exclude, include):
     """Execute scripts with preloaded AiiDA environment."""
     from aiida.cmdline.utils.shell import DEFAULT_MODULES_LIST
-    from aiida.orm import autogroup
+    from aiida.manage import get_manager
+
+    filepath.resolve()
 
     # Prepare the environment for the script to be run
     globals_dict = {
         '__builtins__': globals()['__builtins__'],
         '__name__': '__main__',
-        '__file__': scriptname,
+        '__file__': filepath.name,
         '__doc__': None,
         '__package__': None
     }
@@ -108,48 +100,22 @@ def run(scriptname, varargs, auto_group, auto_group_label_prefix, group_name, ex
     for app_mod, model_name, alias in DEFAULT_MODULES_LIST:
         globals_dict[f'{alias}'] = getattr(__import__(app_mod, {}, {}, model_name), model_name)
 
-    if group_name:
-        warnings.warn('--group-name is deprecated, use `--auto-group-label-prefix` instead', AiidaDeprecationWarning)  # pylint: disable=no-member
-        if auto_group_label_prefix:
-            raise click.BadParameter(
-                'You cannot specify both --group-name and --auto-group-label-prefix; '
-                'use --auto-group-label-prefix only'
-            )
-        auto_group_label_prefix = group_name
-        # To have the old behavior, with auto-group enabled.
-        auto_group = True
-
     if auto_group:
-        aiida_verdilib_autogroup = autogroup.Autogroup()
+        storage_backend = get_manager().get_profile_storage()
+        storage_backend.autogroup.enable()
         # Set the ``group_label_prefix`` if defined, otherwise a default prefix will be used
-        if auto_group_label_prefix is not None:
-            aiida_verdilib_autogroup.set_group_label_prefix(auto_group_label_prefix)
-        aiida_verdilib_autogroup.set_exclude(exclude)
-        aiida_verdilib_autogroup.set_include(include)
-
-        # Note: this is also set in the exec environment! This is the intended behavior
-        autogroup.CURRENT_AUTOGROUP = aiida_verdilib_autogroup
-
-    # Initialize the variable here, otherwise we get UnboundLocalError in the finally clause if it fails to open
-    handle = None
+        storage_backend.autogroup.set_group_label_prefix(auto_group_label_prefix)
+        storage_backend.autogroup.set_exclude(exclude)
+        storage_backend.autogroup.set_include(include)
 
     try:
-        # Here we use a standard open and not open, as exec will later fail if passed a unicode type string.
-        handle = open(scriptname, 'r')
-    except IOError:
-        echo.echo_critical(f"Unable to load file '{scriptname}'")
-    else:
-        try:
-            # Must add also argv[0]
-            argv = [scriptname] + list(varargs)
-            with update_environment(argv=argv):
+        with filepath.open('r', encoding='utf-8') as handle:
+            with update_environment(argv=[str(filepath)] + list(varargs)):
                 # Compile the script for execution and pass it to exec with the globals_dict
-                exec(compile(handle.read(), scriptname, 'exec', dont_inherit=True), globals_dict)  # yapf: disable # pylint: disable=exec-used
-        except SystemExit:  # pylint: disable=try-except-raise
-            # Script called sys.exit()
-            # Re-raise the exception to have the error code properly returned at the end
-            raise
+                exec(compile(handle.read(), str(filepath), 'exec', dont_inherit=True), globals_dict)  # pylint: disable=exec-used
+    except SystemExit:  # pylint: disable=try-except-raise
+        # Script called ``sys.exit()``, re-raise the exception to have the error code properly returned at the end
+        raise
     finally:
-        autogroup.current_autogroup = None
-        if handle:
-            handle.close()
+        storage_backend = get_manager().get_profile_storage()
+        storage_backend.autogroup.disable()
