@@ -7,56 +7,46 @@
 # For further information on the license, see the LICENSE.txt file        #
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
-"""
-.. py:module::conditional
-    :synopsis: Tools for options which are required only if a a set of
-    conditions on the context are fulfilled
-"""
-
+"""Option whose requiredness is determined by a callback function."""
 import click
 
 
 class ConditionalOption(click.Option):
-    """
-    This cli option takes an additional callable parameter and uses that
-    to determine wether a MissingParam should be raised if the option is
-    not given on the cli.
+    """Option whose requiredness is determined by a callback function.
 
-    The callable takes the context as an argument and can look up any
-    amount of other parameter values etc.
+    This option takes an additional callable parameter ``required_fn`` and uses that to determine whether a
+    ``MissingParameter`` exception should be raised if no value is specified for the parameters.
 
-    :param required_fn: callable(ctx) -> True | False, returns True
-        if the parameter is required to have a value.
-        This is typically used when the condition depends on other
-        parameters specified on the command line.
+    The callable should take the context as an argument which it can use to inspect the value of other parameters that
+    have been passed to the command invocation.
+
+    :param required_fn: callable(ctx) -> True | False, returns True if the parameter is required to have a value. This
+        is typically used when the condition depends on other parameters specified on the command line.
     """
 
     def __init__(self, param_decls=None, required_fn=None, **kwargs):
-
-        # note default behaviour for required: False
         self.required_fn = required_fn
 
-        # Required_fn overrides 'required', if defined
+        # If there is not callback to determine requiredness, assume the option is not required.
         if required_fn is not None:
-            # There is a required_fn
-            self.required = False  # So it does not show up as 'required'
+            self.required = False
 
         super().__init__(param_decls=param_decls, **kwargs)
 
-    def full_process_value(self, ctx, value):
+    def process_value(self, ctx, value):
         try:
-            value = super().full_process_value(ctx, value)
-            if self.required_fn and self.value_is_missing(value):
-                if self.is_required(ctx):
-                    raise click.MissingParameter(ctx=ctx, param=self)
+            value = super().process_value(ctx, value)
         except click.MissingParameter:
             if self.is_required(ctx):
                 raise
+        else:
+            if self.required_fn and self.value_is_missing(value) and self.is_required(ctx):
+                raise click.MissingParameter(ctx=ctx, param=self)
+
         return value
 
     def is_required(self, ctx):
         """runs the given check on the context to determine requiredness"""
-
         if self.required_fn:
             return self.required_fn(ctx)
 
