@@ -12,10 +12,11 @@ Plugin for LSF.
 This has been tested on the CERN lxplus cluster (LSF 9.1.3)
 """
 
-import aiida.schedulers
 from aiida.common.escaping import escape_for_bash
+from aiida.common.extendeddicts import AttributeDict
+import aiida.schedulers
 from aiida.schedulers import SchedulerError, SchedulerParsingError
-from aiida.schedulers.datastructures import (JobInfo, JobState, JobResource)
+from aiida.schedulers.datastructures import JobInfo, JobResource, JobState
 
 # This maps LSF status codes to our own state list
 #
@@ -98,11 +99,51 @@ class LsfJobResource(JobResource):
     See https://www-01.ibm.com/support/knowledgecenter/SSETD4_9.1.2/lsf_command_ref/bsub.1.dita?lang=en
     for more details about the parallel environment definition (the -m option of bsub).
     """
-    _default_fields = (
-        'parallel_env',
-        'tot_num_mpiprocs',
-        'default_mpiprocs_per_machine',
-    )
+    _default_fields = ('parallel_env', 'tot_num_mpiprocs', 'default_mpiprocs_per_machine', 'num_machines')
+
+    @classmethod
+    def validate_resources(cls, **kwargs):
+        """Validate the resources against the job resource class of this scheduler.
+
+        :param kwargs: dictionary of values to define the job resources
+        :return: attribute dictionary with the parsed parameters populated
+        :raises ValueError: if the resources are invalid or incomplete
+        """
+        from aiida.common.exceptions import ConfigurationError
+
+        resources = AttributeDict()
+        resources.parallel_env = kwargs.pop('parallel_env', '')
+        resources.use_num_machines = kwargs.pop('use_num_machines', False)
+        num_machines = kwargs.pop('num_machines', None)
+        default_mpiprocs_per_machine = kwargs.pop('default_mpiprocs_per_machine', None)
+
+        if not isinstance(resources.parallel_env, str):
+            raise TypeError('`parallel_env` must be a string')
+
+        if default_mpiprocs_per_machine is not None:
+            raise ConfigurationError('`default_mpiprocs_per_machine` cannot be set.')
+
+        if not resources.use_num_machines and num_machines is not None:
+            raise ConfigurationError('`num_machines` cannot be set unless `use_num_machines` is `True`.')
+
+        if resources.use_num_machines and num_machines is None:
+            raise ConfigurationError('must set `num_machines` when `use_num_machines` is `True`.')
+
+        if resources.use_num_machines and num_machines is not None:
+            try:
+                resources.num_machines = int(num_machines)
+            except (KeyError, ValueError):
+                raise TypeError('`num_machines` must be an integer')
+
+        try:
+            resources.tot_num_mpiprocs = int(kwargs.pop('tot_num_mpiprocs'))
+        except (KeyError, ValueError) as exc:
+            raise TypeError('`tot_num_mpiprocs` must be specified and must be an integer') from exc
+
+        if resources.tot_num_mpiprocs <= 0:
+            raise ValueError('tot_num_mpiprocs must be >= 1')
+
+        return resources
 
     def __init__(self, **kwargs):
         """
@@ -114,28 +155,8 @@ class LsfJobResource(JobResource):
         :raise aiida.common.ConfigurationError: if default_mpiprocs_per_machine was set for this
             computer, since LsfJobResource cannot accept this parameter.
         """
-        from aiida.common.exceptions import ConfigurationError
-        super().__init__()
-
-        self.parallel_env = kwargs.pop('parallel_env', '')
-        if not isinstance(self.parallel_env, str):
-            raise TypeError("When specified, 'parallel_env' must be a string")
-
-        try:
-            self.tot_num_mpiprocs = int(kwargs.pop('tot_num_mpiprocs'))
-        except (KeyError, ValueError) as exc:
-            raise TypeError('tot_num_mpiprocs must be specified and must be an integer') from exc
-
-        default_mpiprocs_per_machine = kwargs.pop('default_mpiprocs_per_machine', None)
-        if default_mpiprocs_per_machine is not None:
-            raise ConfigurationError('default_mpiprocs_per_machine cannot be set for LSF scheduler')
-
-        num_machines = kwargs.pop('num_machines', None)
-        if num_machines is not None:
-            raise ConfigurationError('num_machines cannot be set for LSF scheduler')
-
-        if self.tot_num_mpiprocs <= 0:
-            raise ValueError('tot_num_mpiprocs must be >= 1')
+        resources = self.validate_resources(**kwargs)
+        super().__init__(resources)
 
     def get_tot_num_mpiprocs(self):
         """
@@ -293,10 +314,8 @@ class LsfScheduler(aiida.schedulers.Scheduler):
         :param job_tmpl: an JobTemplate instance with relevant parameters set.
         """
         # pylint: disable=too-many-statements,too-many-branches
-        import string
         import re
-
-        empty_line = ''
+        import string
 
         lines = []
         if job_tmpl.submit_as_hold:
@@ -374,11 +393,14 @@ class LsfScheduler(aiida.schedulers.Scheduler):
         if not job_tmpl.job_resource:
             raise ValueError('Job resources (as the tot_num_mpiprocs) are required for the LSF scheduler plugin')
 
-        lines.append(f'#BSUB -n {job_tmpl.job_resource.get_tot_num_mpiprocs()}')
-        # Note:  make sure that PARALLEL_SCHED_BY_SLOT=Y is NOT
-        # defined in lsb.params (you can check with the output of bparams -l).
-        # Note: the -n option of bsub can also contain a maximum number of
-        # procs to be used
+        if job_tmpl.job_resource.use_num_machines:
+            lines.append(f'#BSUB -nnodes {job_tmpl.job_resource.num_machines}')
+        else:
+            lines.append(f'#BSUB -n {job_tmpl.job_resource.get_tot_num_mpiprocs()}')
+            # Note:  make sure that PARALLEL_SCHED_BY_SLOT=Y is NOT
+            # defined in lsb.params (you can check with the output of bparams -l).
+            # Note: the -n option of bsub can also contain a maximum number of
+            # procs to be used
         if job_tmpl.job_resource.parallel_env:
             lines.append(f'#BSUB -m "{job_tmpl.job_resource.parallel_env}"')
 
@@ -408,9 +430,7 @@ class LsfScheduler(aiida.schedulers.Scheduler):
                     raise ValueError
             except ValueError as exc:
                 raise ValueError(
-                    'max_memory_kb must be '
-                    "a positive integer (in kB)! It is instead '{}'"
-                    ''.format((job_tmpl.max_memory_kb))
+                    f'max_memory_kb must be a positive integer (in kB)! It is instead `{job_tmpl.max_memory_kb}`'
                 ) from exc
             # The -M option sets a per-process (soft) memory limit for all the
             # processes that belong to this job
@@ -419,22 +439,8 @@ class LsfScheduler(aiida.schedulers.Scheduler):
         if job_tmpl.custom_scheduler_commands:
             lines.append(job_tmpl.custom_scheduler_commands)
 
-        # Job environment variables are to be set on one single line.
-        # This is a tough job due to the escaping of commas, etc.
-        # moreover, I am having issues making it work.
-        # Therefore, I assume that this is bash and export variables by
-        # hand.
         if job_tmpl.job_environment:
-            lines.append(empty_line)
-            lines.append('# ENVIRONMENT VARIABLES BEGIN ###')
-            if not isinstance(job_tmpl.job_environment, dict):
-                raise ValueError('If you provide job_environment, it must be a dictionary')
-            for key, value in job_tmpl.job_environment.items():
-                lines.append(f'export {key.strip()}={escape_for_bash(value)}')
-            lines.append('# ENVIRONMENT VARIABLES END  ###')
-            lines.append(empty_line)
-
-        lines.append(empty_line)
+            lines.append(self._get_submit_script_environment_variables(job_tmpl))
 
         # The following seems to be the only way to copy the input files
         # to the node where the computation are actually launched (the
@@ -540,10 +546,7 @@ fi
             try:
                 job_state_string = _MAP_STATUS_LSF[job_state_raw]
             except KeyError:
-                self.logger.warning(
-                    "Unrecognized job_state '{}' for job "
-                    'id {}'.format(job_state_raw, this_job.job_id)
-                )
+                self.logger.warning(f"Unrecognized job_state '{job_state_raw}' for job id {this_job.job_id}")
                 job_state_string = JobState.UNDETERMINED
 
             this_job.job_state = job_state_string
