@@ -11,13 +11,15 @@
 # pylint: disable=missing-function-docstring
 from contextlib import contextmanager, nullcontext
 import functools
+import gc
+import pathlib
 from typing import TYPE_CHECKING, Iterator, List, Optional, Sequence, Set, Union
 
 from disk_objectstore import Container
 from sqlalchemy import table
 from sqlalchemy.orm import Session, scoped_session, sessionmaker
 
-from aiida.common.exceptions import ClosedStorage, IntegrityError
+from aiida.common.exceptions import ClosedStorage, ConfigurationError, IntegrityError
 from aiida.manage.configuration.profile import Profile
 from aiida.orm.entities import EntityTypes
 from aiida.orm.implementation import BackendEntity, StorageBackend
@@ -40,6 +42,28 @@ CONTAINER_DEFAULTS: dict = {
 }
 
 
+def get_filepath_container(profile: Profile) -> pathlib.Path:
+    """Return the filepath of the disk-object store container."""
+    from urllib.parse import urlparse
+
+    try:
+        parts = urlparse(profile.storage_config['repository_uri'])
+    except KeyError:
+        raise KeyError(f'invalid profile {profile.name}: `repository_uri` not defined in `storage.config`.')
+
+    if parts.scheme != 'file':
+        raise ConfigurationError(
+            f'invalid profile {profile.name}: `storage.config.repository_uri` does not start with `file://`.'
+        )
+
+    filepath = pathlib.Path(parts.path)
+
+    if not filepath.is_absolute():
+        raise ConfigurationError(f'invalid profile {profile.name}: `storage.config.repository_uri` is not absolute')
+
+    return filepath.expanduser() / 'container'
+
+
 class PsqlDosBackend(StorageBackend):  # pylint: disable=too-many-public-methods
     """An AiiDA storage backend that stores data in a PostgreSQL database and disk-objectstore repository.
 
@@ -121,6 +145,10 @@ class PsqlDosBackend(StorageBackend):  # pylint: disable=too-many-public-methods
         self._session_factory.close()
         self._session_factory = None
 
+        # Without this, sqlalchemy keeps a weakref to a session
+        # in sqlalchemy.orm.session._sessions
+        gc.collect()
+
     def _clear(self, recreate_user: bool = True) -> None:
         from aiida.storage.psql_dos.models.settings import DbSetting
         from aiida.storage.psql_dos.models.user import DbUser
@@ -149,13 +177,14 @@ class PsqlDosBackend(StorageBackend):  # pylint: disable=too-many-public-methods
             ):
                 session.execute(table(table_name).delete())
             session.expunge_all()
+            self._default_user = None
 
             # restore the default user
             if recreate_user and default_user_kwargs:
                 session.add(DbUser(**default_user_kwargs))
 
         # Clear the repository and reset the repository UUID
-        container = Container(self.profile.repository_path / 'container')
+        container = Container(str(get_filepath_container(self.profile)))
         container.init_container(clear=True, **CONTAINER_DEFAULTS)
         container_id = container.container_id
         with self.transaction():
@@ -165,8 +194,7 @@ class PsqlDosBackend(StorageBackend):  # pylint: disable=too-many-public-methods
 
     def get_repository(self) -> 'DiskObjectStoreRepositoryBackend':
         from aiida.repository.backend import DiskObjectStoreRepositoryBackend
-
-        container = Container(self.profile.repository_path / 'container')
+        container = Container(str(get_filepath_container(self.profile)))
         return DiskObjectStoreRepositoryBackend(container=container)
 
     @property
