@@ -9,11 +9,11 @@
 ###########################################################################
 """Transport interface."""
 import abc
+from collections import OrderedDict
+import fnmatch
 import os
 import re
-import fnmatch
 import sys
-from collections import OrderedDict
 
 from aiida.common.exceptions import InternalError
 from aiida.common.lang import classproperty
@@ -33,11 +33,16 @@ def validate_positive_number(ctx, param, value):  # pylint: disable=unused-argum
         from click import BadParameter
         raise BadParameter(f'{value} is not a valid positive number')
 
+    return value
+
 
 class Transport(abc.ABC):
-    """Abstract class for a generic transport (ssh, local, ...) ontains the set of minimal methods."""
+    """Abstract class for a generic transport (ssh, local, ...) contains the set of minimal methods."""
     # pylint: disable=too-many-public-methods
 
+    # This will be used for ``Computer.get_minimum_job_poll_interval``
+    DEFAULT_MINIMUM_JOB_POLL_INTERVAL = 10
+
     # This is used as a global default in case subclasses don't redefine this,
     # but this should  be redefined in plugins where appropriate
     _DEFAULT_SAFE_OPEN_INTERVAL = 30.
@@ -96,6 +101,9 @@ class Transport(abc.ABC):
         self._is_open = False
         self._enters = 0
 
+        # for accessing the identity of the underlying machine
+        self.hostname = kwargs.get('machine')
+
     def __enter__(self):
         """
         For transports that require opening a connection, opens
@@ -138,17 +146,17 @@ class Transport(abc.ABC):
     def is_open(self):
         return self._is_open
 
+    @abc.abstractmethod
     def open(self):
         """
         Opens a local transport channel
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def close(self):
         """
         Closes the local transport channel
         """
-        raise NotImplementedError
 
     def __repr__(self):
         return f'<{self.__class__.__name__}: {str(self)}>'
@@ -161,12 +169,12 @@ class Transport(abc.ABC):
         """
         Pass the data that should be passed automatically to self.logger
         as 'extra' keyword. This is typically useful if you pass data
-        obtained using get_dblogger_extra in aiida.backends.djsite.utils, to automatically
+        obtained using get_dblogger_extra in aiida.orm.utils.log, to automatically
         log also to the DbLog table.
 
         :param logger_extra: data that you want to pass as extra to the
           self.logger. To write to DbLog, it should be created by the
-          aiida.backends.djsite.utils.get_dblogger_extra function. Pass None if you
+          aiida.orm.utils.log.get_dblogger_extra function. Pass None if you
           do not want to have extras passed.
         """
         self._logger_extra = logger_extra
@@ -186,21 +194,6 @@ class Transport(abc.ABC):
             return doclines[0].strip()
         return 'No documentation available'
 
-    @classmethod
-    def get_valid_transports(cls):
-        """Return the list of registered transport entry points.
-
-        .. deprecated:: 1.4.0
-
-            Will be removed in `2.0.0`, use `aiida.plugins.entry_point.get_entry_point_names` instead
-        """
-        import warnings
-        from aiida.common.warnings import AiidaDeprecationWarning
-        from aiida.plugins.entry_point import get_entry_point_names
-        message = 'method is deprecated, use `aiida.plugins.entry_point.get_entry_point_names` instead'
-        warnings.warn(message, AiidaDeprecationWarning)  # pylint: disable=no-member
-        return get_entry_point_names('aiida.transports')
-
     @classmethod
     def get_valid_auth_params(cls):
         """
@@ -209,10 +202,10 @@ class Transport(abc.ABC):
         if cls._valid_auth_options is None:
             raise NotImplementedError
         else:
-            return cls.auth_options.keys()  # pylint: disable=no-member
+            return cls.auth_options.keys()
 
     @classproperty
-    def auth_options(cls):  # pylint: disable=no-self-argument
+    def auth_options(cls) -> OrderedDict:  # pylint: disable=no-self-argument
         """Return the authentication options to be used for building the CLI.
 
         :return: `OrderedDict` of tuples, with first element option name and second dictionary of kwargs
@@ -270,6 +263,7 @@ class Transport(abc.ABC):
         """
         return self._safe_open_interval
 
+    @abc.abstractmethod
     def chdir(self, path):
         """
         Change directory to 'path'
@@ -279,8 +273,7 @@ class Transport(abc.ABC):
         :rtype: str
         """
 
-        raise NotImplementedError
-
+    @abc.abstractmethod
     def chmod(self, path, mode):
         """
         Change permissions of a path.
@@ -288,8 +281,8 @@ class Transport(abc.ABC):
         :param str path: path to file
         :param int mode: new permissions
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def chown(self, path, uid, gid):
         """
         Change the owner (uid) and group (gid) of a file.
@@ -301,8 +294,8 @@ class Transport(abc.ABC):
         :param int uid: new owner's uid
         :param int gid: new group id
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def copy(self, remotesource, remotedestination, dereference=False, recursive=True):
         """
         Copy a file or a directory from remote source to remote destination
@@ -317,8 +310,8 @@ class Transport(abc.ABC):
 
         :raises: IOError, if one of src or dst does not exist
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def copyfile(self, remotesource, remotedestination, dereference=False):
         """
         Copy a file from remote source to remote destination
@@ -331,8 +324,8 @@ class Transport(abc.ABC):
 
         :raises IOError: if one of src or dst does not exist
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def copytree(self, remotesource, remotedestination, dereference=False):
         """
         Copy a folder from remote source to remote destination
@@ -345,7 +338,6 @@ class Transport(abc.ABC):
 
         :raise IOError: if one of src or dst does not exist
         """
-        raise NotImplementedError
 
     def copy_from_remote_to_remote(self, transportdestination, remotesource, remotedestination, **kwargs):
         """
@@ -397,6 +389,7 @@ class Transport(abc.ABC):
             for filename in sandbox.get_content_list():
                 transportdestination.put(os.path.join(sandbox.abspath, filename), remotedestination, **kwargs_put)
 
+    @abc.abstractmethod
     def _exec_command_internal(self, command, **kwargs):
         """
         Execute the command on the shell, similarly to os.system.
@@ -411,21 +404,47 @@ class Transport(abc.ABC):
         :return: stdin, stdout, stderr and the session, when this exists \
                  (can be None).
         """
-        raise NotImplementedError
 
-    def exec_command_wait(self, command, **kwargs):
+    @abc.abstractmethod
+    def exec_command_wait_bytes(self, command, stdin=None, **kwargs):
         """
         Execute the command on the shell, waits for it to finish,
-        and return the retcode, the stdout and the stderr.
+        and return the retcode, the stdout and the stderr as bytes.
 
-        Enforce the execution to be run from the pwd (as given by
-        self.getcwd), if this is not None.
+        Enforce the execution to be run from the pwd (as given by self.getcwd), if this is not None.
+
+        The command implementation can have some additional plugin-specific kwargs.
 
         :param str command: execute the command given as a string
-        :return: a list: the retcode (int), stdout (str) and stderr (str).
+        :param stdin: (optional,default=None) can be a string or a file-like object.
+        :return: a tuple: the retcode (int), stdout (bytes) and stderr (bytes).
         """
-        raise NotImplementedError
 
+    def exec_command_wait(self, command, stdin=None, encoding='utf-8', **kwargs):
+        """
+        Executes the specified command and waits for it to finish.
+
+        :note: this function also decodes the bytes received into a string with the specified encoding,
+            which is set to be ``utf-8`` by default (for backward-compatibility with earlier versions) of
+            AiiDA.
+            Use this method only if you are sure that you are getting a properly encoded string; otherwise,
+            use the ``exec_command_wait_bytes`` method that returns the undecoded byte stream.
+
+        :note: additional kwargs are passed to the ``exec_command_wait_bytes`` function, that might use them
+            depending on the plugin.
+
+        :param command: the command to execute
+        :param stdin: (optional,default=None) can be a string or a file-like object.
+        :param encoding: the encoding to use to decode the byte stream received from the remote command execution.
+
+        :return: a tuple with (return_value, stdout, stderr) where stdout and stderr are both strings, decoded
+            with the specified encoding.
+        """
+        retval, stdout_bytes, stderr_bytes = self.exec_command_wait_bytes(command=command, stdin=stdin, **kwargs)
+        # Return the decoded strings
+        return (retval, stdout_bytes.decode(encoding), stderr_bytes.decode(encoding))
+
+    @abc.abstractmethod
     def get(self, remotepath, localpath, *args, **kwargs):
         """
         Retrieve a file or folder from remote source to local destination
@@ -434,8 +453,8 @@ class Transport(abc.ABC):
         :param remotepath: (str) remote_folder_path
         :param localpath: (str) local_folder_path
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def getfile(self, remotepath, localpath, *args, **kwargs):
         """
         Retrieve a file from remote source to local destination
@@ -444,8 +463,8 @@ class Transport(abc.ABC):
         :param str remotepath: remote_folder_path
         :param str localpath: local_folder_path
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def gettree(self, remotepath, localpath, *args, **kwargs):
         """
         Retrieve a folder recursively from remote source to local destination
@@ -454,16 +473,16 @@ class Transport(abc.ABC):
         :param str remotepath: remote_folder_path
         :param str localpath: local_folder_path
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def getcwd(self):
         """
         Get working directory
 
         :return: a string identifying the current working directory
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def get_attribute(self, path):
         """
         Return an object FixedFieldsAttributeDict for file in a given path,
@@ -485,7 +504,6 @@ class Transport(abc.ABC):
         :param str path: path to file
         :return: object FixedFieldsAttributeDict
         """
-        raise NotImplementedError
 
     def get_mode(self, path):
         """
@@ -498,6 +516,7 @@ class Transport(abc.ABC):
 
         return stat.S_IMODE(self.get_attribute(path).st_mode)
 
+    @abc.abstractmethod
     def isdir(self, path):
         """
         True if path is an existing directory.
@@ -505,8 +524,8 @@ class Transport(abc.ABC):
         :param str path: path to directory
         :return: boolean
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def isfile(self, path):
         """
         Return True if path is an existing file.
@@ -514,8 +533,8 @@ class Transport(abc.ABC):
         :param str path: path to file
         :return: boolean
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def listdir(self, path='.', pattern=None):
         """
         Return a list of the names of the entries in the given path.
@@ -527,7 +546,6 @@ class Transport(abc.ABC):
                             filters in Unix style. Unix only.
         :return: a list of strings
         """
-        raise NotImplementedError
 
     def listdir_withattributes(self, path='.', pattern=None):  # pylint: disable=unused-argument
         """
@@ -560,6 +578,7 @@ class Transport(abc.ABC):
             retlist.append({'name': file_name, 'attributes': attributes, 'isdir': self.isdir(filepath)})
         return retlist
 
+    @abc.abstractmethod
     def makedirs(self, path, ignore_existing=False):
         """
         Super-mkdir; create a leaf directory and all intermediate ones.
@@ -572,8 +591,8 @@ class Transport(abc.ABC):
 
         :raises: OSError, if directory at path already exists
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def mkdir(self, path, ignore_existing=False):
         """
         Create a folder (directory) named path.
@@ -584,8 +603,8 @@ class Transport(abc.ABC):
 
         :raises: OSError, if directory at path already exists
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def normalize(self, path='.'):
         """
         Return the normalized path (on the server) of a given path.
@@ -596,8 +615,8 @@ class Transport(abc.ABC):
 
         :raise IOError: if the path can't be resolved on the server
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def put(self, localpath, remotepath, *args, **kwargs):
         """
         Put a file or a directory from local src to remote dst.
@@ -607,8 +626,8 @@ class Transport(abc.ABC):
         :param str localpath: absolute path to local source
         :param str remotepath: path to remote destination
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def putfile(self, localpath, remotepath, *args, **kwargs):
         """
         Put a file from local src to remote dst.
@@ -617,8 +636,8 @@ class Transport(abc.ABC):
         :param str localpath: absolute path to local file
         :param str remotepath: path to remote file
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def puttree(self, localpath, remotepath, *args, **kwargs):
         """
         Put a folder recursively from local src to remote dst.
@@ -627,8 +646,8 @@ class Transport(abc.ABC):
         :param str localpath: absolute path to local folder
         :param str remotepath: path to remote folder
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def remove(self, path):
         """
         Remove the file at the given path. This only works on files;
@@ -638,8 +657,8 @@ class Transport(abc.ABC):
 
         :raise IOError: if the path is a directory
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def rename(self, oldpath, newpath):
         """
         Rename a file or folder from oldpath to newpath.
@@ -650,8 +669,8 @@ class Transport(abc.ABC):
         :raises IOError: if oldpath/newpath is not found
         :raises ValueError: if oldpath/newpath is not a valid string
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def rmdir(self, path):
         """
         Remove the folder named path.
@@ -659,16 +678,16 @@ class Transport(abc.ABC):
 
         :param str path: absolute path to the folder to remove
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def rmtree(self, path):
         """
         Remove recursively the content at path
 
         :param str path: absolute path to remove
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def gotocomputer_command(self, remotedir):
         """
         Return a string to be run using os.system in order to connect
@@ -682,8 +701,8 @@ class Transport(abc.ABC):
 
         :param str remotedir: the full path of the remote directory
         """
-        raise NotImplementedError
 
+    @abc.abstractmethod
     def symlink(self, remotesource, remotedestination):
         """
         Create a symbolic link between the remote source and the remote
@@ -692,7 +711,6 @@ class Transport(abc.ABC):
         :param remotesource: remote source
         :param remotedestination: remote destination
         """
-        raise NotImplementedError
 
     def whoami(self):
         """
@@ -704,6 +722,8 @@ class Transport(abc.ABC):
         """
 
         command = 'whoami'
+        # Assuming here that the username is either ASCII or UTF-8 encoded
+        # This should be true essentially always
         retval, username, stderr = self.exec_command_wait(command)
         if retval == 0:
             if stderr.strip():
@@ -713,11 +733,11 @@ class Transport(abc.ABC):
         self.logger.error(f"Problem executing whoami. Exit code: {retval}, stdout: '{username}', stderr: '{stderr}'")
         raise IOError(f'Error while executing whoami. Exit code: {retval}')
 
+    @abc.abstractmethod
     def path_exists(self, path):
         """
         Returns True if path exists, False otherwise.
         """
-        raise NotImplementedError
 
     # The following definitions are almost copied and pasted
     # from the python module glob.
